@@ -40,6 +40,6 @@ gboolean general_enabled(gint i);
 gchar *general_stringify_enabled_list(void);
 void general_enable_from_stringified_list(const gchar * list);
 
-extern GeneralPluginData *gp_data;
+extern GeneralPluginData gp_data;
 
 #endif
