@@ -47,6 +47,7 @@
 #include "libbeep/configdb.h"
 #include "libbeep/beepctrl.h"
 #include "libbeep/util.h"
+#include "libbeep/vfs.h"
 
 #include "controlsocket.h"
 #include "dnd.h"
@@ -299,13 +300,7 @@ static bmp_cfg_strent bmp_strents[] = {
 
 static gint ncfgsent = G_N_ELEMENTS(bmp_strents);
 
-gchar *bmp_user_dir = NULL;
-gchar *bmp_user_plugin_dir = NULL;
-gchar *bmp_user_skin_dir = NULL;
-gchar *bmp_user_skin_thumb_dir = NULL;
-gchar *bmp_config_filename = NULL;
-gchar *bmp_playlist_filename = NULL;
-gchar *bmp_log_filename = NULL;
+gchar *bmp_paths[BMP_PATH_COUNT] = {};
 
 GList *dock_window_list = NULL;
 
@@ -382,53 +377,40 @@ bmp_make_user_dir(void)
 {
     const mode_t mode755 = S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
 
-    make_directory(bmp_user_dir, mode755);
-    make_directory(bmp_user_skin_dir, mode755);
-    make_directory(bmp_user_skin_thumb_dir, mode755);
-    make_directory(bmp_user_plugin_dir, mode755);
+    make_directory(bmp_paths[BMP_PATH_USER_DIR], mode755);
+    make_directory(bmp_paths[BMP_PATH_USER_PLUGIN_DIR], mode755);
+    make_directory(bmp_paths[BMP_PATH_USER_SKIN_DIR], mode755);
+    make_directory(bmp_paths[BMP_PATH_SKIN_THUMB_DIR], mode755);
 }
 
 static void
 bmp_free_paths(void)
 {
-    g_free(bmp_user_dir);
-    g_free(bmp_user_plugin_dir);
-    g_free(bmp_user_skin_dir);
-    g_free(bmp_user_skin_thumb_dir);
-    g_free(bmp_config_filename);
-    g_free(bmp_playlist_filename);
-    g_free(bmp_log_filename);
+    int i;
+
+    for (i = 0; i < BMP_PATH_COUNT; i++)
+    {
+        g_free(bmp_paths[i]);
+        bmp_paths[i] = 0;
+    }
 }
 
 
+#define USER_PATH(path) \
+    g_build_filename(bmp_paths[BMP_PATH_USER_DIR], path, NULL);
+
 static void
 bmp_init_paths(void)
 {
-    bmp_user_dir = g_build_filename(g_get_home_dir(), BMP_RCPATH, NULL);
+    bmp_paths[BMP_PATH_USER_DIR] = g_build_filename(g_get_home_dir(), BMP_RCPATH, NULL);
 
-    bmp_user_plugin_dir = g_build_filename(bmp_user_dir,
-                                           BMP_USER_PLUGIN_DIR_BASENAME,
-                                           NULL);
-
-    bmp_user_skin_dir = g_build_filename(bmp_user_dir,
-                                         BMP_SKIN_DIR_BASENAME,
-                                         NULL);
-
-    bmp_config_filename = g_build_filename(bmp_user_dir,
-                                           BMP_CONFIG_BASENAME,
-                                           NULL);
-
-    bmp_playlist_filename = g_build_filename(bmp_user_dir,
-                                             BMP_PLAYLIST_BASENAME,
-                                             NULL);
-
-    bmp_log_filename = g_build_filename(bmp_user_dir,
-                                        BMP_LOG_BASENAME,
-                                        NULL);
-
-    bmp_user_skin_thumb_dir = g_build_filename(bmp_user_skin_dir,
-                                               BMP_SKIN_THUMB_DIR_BASENAME,
-                                               NULL);
+    bmp_paths[BMP_PATH_USER_PLUGIN_DIR] = USER_PATH(BMP_USER_PLUGIN_DIR_BASENAME);
+    bmp_paths[BMP_PATH_USER_SKIN_DIR] = USER_PATH(BMP_SKIN_DIR_BASENAME);
+    bmp_paths[BMP_PATH_SKIN_THUMB_DIR] = USER_PATH(BMP_SKIN_THUMB_DIR_BASENAME);
+    bmp_paths[BMP_PATH_CONFIG_FILE] = USER_PATH(BMP_CONFIG_BASENAME);
+    bmp_paths[BMP_PATH_PLAYLIST_FILE] = USER_PATH(BMP_PLAYLIST_BASENAME);
+    bmp_paths[BMP_PATH_ACCEL_FILE] = USER_PATH(BMP_ACCEL_BASENAME);
+    bmp_paths[BMP_PATH_LOG_FILE] = USER_PATH(BMP_LOG_BASENAME);
 
     g_atexit(bmp_free_paths);
 }
@@ -624,7 +606,7 @@ bmp_config_save(void)
 
     bmp_cfg_db_close(db);
 
-    playlist_save(bmp_playlist_filename, FALSE);
+    playlist_save(bmp_paths[BMP_PATH_PLAYLIST_FILE], FALSE);
 }
 
 static void
@@ -869,7 +851,7 @@ segfault_handler(gint sig)
 static void
 bmp_setup_logger(void)
 {
-    if (!bmp_logger_start(bmp_log_filename))
+    if (!bmp_logger_start(bmp_paths[BMP_PATH_LOG_FILE]))
         return;
 
     g_atexit(bmp_logger_stop);
@@ -921,7 +903,7 @@ main(gint argc, gchar ** argv)
         exit(EXIT_FAILURE);
     }
 
-    g_set_application_name(gettext(application_name));
+    g_set_application_name(_(application_name));
 
     g_thread_init(NULL);
     if (!g_thread_supported()) {
@@ -946,15 +928,11 @@ main(gint argc, gchar ** argv)
         exit(EXIT_SUCCESS);
     }
 
-#ifdef HAVE_GNOME_VFS
-    if (!gnome_vfs_init()) {
-        g_printerr(_("Could not initialize gnome-vfs.\n"));
+    if (!vfs_init()) {
+        g_printerr(Q_("Could not initialize VFS.\n"));
         exit(EXIT_FAILURE);
     }
 
-    g_atexit(gnome_vfs_shutdown);
-#endif
-
     signal(SIGPIPE, SIG_IGN);   /* for controlsocket.c */
     signal(SIGSEGV, segfault_handler);
 
@@ -973,6 +951,8 @@ main(gint argc, gchar ** argv)
 
     bmp_set_default_icon();
 
+    gtk_accel_map_load(bmp_paths[BMP_PATH_ACCEL_FILE]);
+
     mainwin_create();
     playlistwin_create();
     equalizerwin_create();
@@ -987,7 +967,7 @@ main(gint argc, gchar ** argv)
     plugin_system_init();
     read_volume(VOLSET_STARTUP);
 
-    playlist_load(bmp_playlist_filename);
+    playlist_load(bmp_paths[BMP_PATH_PLAYLIST_FILE]);
     playlist_set_position(cfg.playlist_position);
 
     /* this needs to be called after all 3 windows are created and
