@@ -28,9 +28,9 @@
 #include "widget.h"
 
 void
-monostereo_draw(Widget * w)
+monostereo_draw(Widget * widget)
 {
-    MonoStereo *ms = (MonoStereo *) w;
+    MonoStereo *ms = (MonoStereo *) widget;
     GdkPixmap *obj;
 
     obj = ms->ms_widget.parent;
@@ -64,15 +64,19 @@ monostereo_draw(Widget * w)
 }
 
 void
-monostereo_set_num_channels(MonoStereo * ms, gint nch)
+monostereo_set_num_channels(MonoStereo * ms,
+                            gint nch)
 {
     ms->ms_num_channels = nch;
-    draw_widget(ms);
+    widget_draw(WIDGET(ms));
 }
 
 MonoStereo *
-create_monostereo(GList ** wlist, GdkPixmap * parent,
-                  GdkGC * gc, gint x, gint y, SkinPixmapId si)
+create_monostereo(GList ** wlist,
+                  GdkPixmap * parent,
+                  GdkGC * gc,
+                  gint x, gint y, 
+                  SkinPixmapId si)
 {
     MonoStereo *ms;
 
@@ -81,6 +85,6 @@ create_monostereo(GList ** wlist, GdkPixmap * parent,
     ms->ms_widget.draw = monostereo_draw;
     ms->ms_skin_index = si;
 
-    add_widget(wlist, ms);
+    widget_list_add(wlist, WIDGET(ms));
     return ms;
 }
