@@ -55,21 +55,24 @@
 #include "urldecode.h"
 #include "util.h"
 
-#include "pixmaps.h"
-
 #include "debug.h"
 
-
-typedef gint(*PlaylistCompareFunc) (PlaylistEntry * a, PlaylistEntry * b);
-
-
-GList *playlist = NULL;
-GList *shuffle_list = NULL;
-GList *queued_list = NULL;
+typedef gint (*PlaylistCompareFunc) (const PlaylistEntry * a, const PlaylistEntry * b);
+typedef void (*PlaylistSaveFunc) (FILE * file);
 
 PlaylistEntry *playlist_position;
-
 G_LOCK_DEFINE(playlist);
+
+/* NOTE: match the order listed in PlaylistFormat enum */
+static const gchar *playlist_format_suffixes[] = { 
+    ".m3u", ".pls", NULL 
+};
+
+static GList *playlist = NULL;
+static GList *shuffle_list = NULL;
+static GList *queued_list = NULL;
+
+
 G_LOCK_DEFINE(playlist_get_info_going);
 
 static gchar *playlist_current_name = NULL;
@@ -78,11 +81,12 @@ static gboolean playlist_get_info_scan_active = FALSE;
 static gboolean playlist_get_info_going = FALSE;
 static GThread *playlist_get_info_thread;
 
+
 static gint path_compare(const gchar * a, const gchar * b);
-static gint playlist_compare_path(PlaylistEntry * a, PlaylistEntry * b);
-static gint playlist_compare_filename(PlaylistEntry * a, PlaylistEntry * b);
-static gint playlist_compare_title(PlaylistEntry * a, PlaylistEntry * b);
-static gint playlist_compare_date(PlaylistEntry * a, PlaylistEntry * b);
+static gint playlist_compare_path(const PlaylistEntry * a, const PlaylistEntry * b);
+static gint playlist_compare_filename(const PlaylistEntry * a, const PlaylistEntry * b);
+static gint playlist_compare_title(const PlaylistEntry * a, const PlaylistEntry * b);
+static gint playlist_compare_date(const PlaylistEntry * a, const PlaylistEntry * b);
 
 static PlaylistCompareFunc playlist_compare_func_table[] = {
     playlist_compare_path,
@@ -91,6 +95,14 @@ static PlaylistCompareFunc playlist_compare_func_table[] = {
     playlist_compare_date
 };
 
+static void playlist_save_m3u(FILE * file);
+static void playlist_save_pls(FILE * file);
+
+static PlaylistSaveFunc playlist_save_func_table[] = {
+    playlist_save_m3u,
+    playlist_save_pls
+};
+
 
 static guint playlist_load_ins(const gchar * filename, gint pos);
 
@@ -107,7 +119,8 @@ static void playlist_recalc_total_time(void);
 
 PlaylistEntry *
 playlist_entry_new(const gchar * filename,
-                   const gchar * title, const gint length)
+                   const gchar * title,
+                   const gint length)
 {
     PlaylistEntry *entry;
 
@@ -426,14 +439,26 @@ __playlist_ins(const gchar * filename, gint pos)
     playlist_recalc_total_time();
 }
 
-gboolean
-is_playlist_name(const gchar * pathname)
+
+PlaylistFormat
+playlist_format_get_from_name(const gchar * filename)
 {
-    static gchar *suffixes[] = { ".m3u", ".pls", NULL };
+    int i;
 
-    g_return_val_if_fail(pathname != NULL, FALSE);
+    for (i = 0; i < PLAYLIST_FORMAT_COUNT; i++)
+    {
+        if (str_has_suffix_nocase(filename, playlist_format_suffixes[i]))
+            return i;
+    }
 
-    return str_has_suffixes_nocase(pathname, suffixes);
+    return PLAYLIST_FORMAT_UNKNOWN;
+}
+
+gboolean
+is_playlist_name(const gchar * filename)
+{
+    g_return_val_if_fail(filename != NULL, FALSE);
+    return playlist_format_get_from_name(filename) != PLAYLIST_FORMAT_UNKNOWN;
 }
 
 
@@ -495,6 +520,21 @@ typedef struct {
     ino_t ino;
 } DeviceInode;
 
+static DeviceInode *
+devino_new(dev_t device,
+           ino_t inode)
+{
+    DeviceInode *devino = g_new0(DeviceInode, 1);
+
+    if (devino)
+    {
+        devino->dev = device;
+        devino->ino = inode;
+    }
+
+    return devino;
+}
+
 static guint
 devino_hash(gconstpointer key)
 {
@@ -503,19 +543,30 @@ devino_hash(gconstpointer key)
 }
 
 static gint
-devino_compare(gconstpointer a, gconstpointer b)
+devino_compare(gconstpointer a,
+               gconstpointer b)
 {
     const DeviceInode *da = a, *db = b;
     return (da->dev == db->dev && da->ino == db->ino);
 }
 
 static gboolean
-devino_destroy(gpointer key, gpointer value, gpointer data)
+devino_destroy(gpointer key, 
+               gpointer value,
+               gpointer data)
 {
     g_free(key);
     return TRUE;
 }
 
+static gboolean
+file_is_hidden(const gchar * filename)
+{
+    // FIXME: remove the const cast
+    g_return_val_if_fail(filename != NULL, FALSE);
+    return (g_basename((gchar *) filename)[0] == '.');
+}
+
 static GList *
 playlist_dir_find_files(const gchar * path,
                         gboolean background,
@@ -532,9 +583,7 @@ playlist_dir_find_files(const gchar * path,
         return NULL;
 
     stat(path, &statbuf);
-    devino = g_new0(DeviceInode, 1);
-    devino->dev = statbuf.st_dev;
-    devino->ino = statbuf.st_ino;
+    devino = devino_new(statbuf.st_dev, statbuf.st_ino);
 
     if (g_hash_table_lookup(htab, devino)) {
         g_free(devino);
@@ -560,7 +609,7 @@ playlist_dir_find_files(const gchar * path,
     while ((dir_entry = g_dir_read_name(dir))) {
         gchar *filename;
 
-        if (dir_entry[0] == '.')
+        if (file_is_hidden(dir_entry))
             continue;
 
         filename = g_build_filename(path, dir_entry, NULL);
@@ -584,10 +633,29 @@ playlist_dir_find_files(const gchar * path,
     return list;
 }
 
-guint
-playlist_ins_dir(const gchar * path, gint pos, gboolean background)
+gboolean
+playlist_add(const gchar * filename)
 {
+    return playlist_ins(filename, -1);
+}
 
+guint 
+playlist_add_dir(const gchar * directory)
+{
+    return playlist_ins_dir(directory, -1, TRUE);
+}
+
+guint
+playlist_add_url(const gchar * url)
+{
+    return playlist_ins_url(url, -1);
+}
+
+guint
+playlist_ins_dir(const gchar * path,
+                    gint pos,
+                    gboolean background)
+{
     guint entries = 0;
     GList *list, *node;
     GHashTable *htab;
@@ -616,7 +684,8 @@ playlist_ins_dir(const gchar * path, gint pos, gboolean background)
 }
 
 guint
-playlist_ins_url(const gchar * string, gint pos)
+playlist_ins_url(const gchar * string,
+                    gint pos)
 {
     gchar *tmp;
     gint i = 1, entries = 0;
@@ -704,7 +773,7 @@ playlist_set_info(const gchar * title, gint length, gint rate,
 void
 playlist_check_pos_current(void)
 {
-    guint pos, row, bottom;
+    gint pos, row, bottom;
 
     PLAYLIST_LOCK();
     if (!playlist || !playlist_position || !playlistwin_list) {
@@ -909,6 +978,19 @@ playlist_clear_queue(void)
     playlistwin_update_list();
 }
 
+void
+playlist_queue_remove(guint pos)
+{
+    void *entry;
+
+    PLAYLIST_LOCK();
+    entry = g_list_nth_data(playlist, pos);
+    queued_list = g_list_remove(queued_list, entry);
+    PLAYLIST_UNLOCK();
+
+    playlistwin_update_list();
+}
+
 gint
 playlist_get_queue_position(PlaylistEntry * entry)
 {
@@ -1019,6 +1101,18 @@ playlist_get_length(void)
     return retval;
 }
 
+gint
+playlist_queue_get_length(void)
+{
+    gint length;
+
+    PLAYLIST_LOCK();
+    length = g_list_length(queued_list);
+    PLAYLIST_UNLOCK();
+
+    return length;
+}
+
 gint
 playlist_get_length_nolock(void)
 {
@@ -1089,32 +1183,20 @@ playlist_get_current_length(void)
     return len;
 }
 
-gboolean
-playlist_save(const gchar * filename, gboolean is_pls)
+static void
+playlist_save_m3u(FILE * file)
 {
     GList *node;
-    FILE *file;
 
-    playlist_set_current_name(filename);
+    g_return_if_fail(file != NULL);
 
-    if ((file = fopen(filename, "w")) == NULL)
-        return FALSE;
-
-    if (is_pls) {
-        g_fprintf(file, "[playlist]\n");
-        g_fprintf(file, "NumberOfEntries=%d\n", playlist_get_length());
-    }
-    else if (cfg.use_pl_metadata)
+    if (cfg.use_pl_metadata)
         g_fprintf(file, "#EXTM3U\n");
 
     PLAYLIST_LOCK();
+
     for (node = playlist; node; node = g_list_next(node)) {
-        PlaylistEntry *entry = node->data;
-        if (is_pls) {
-            g_fprintf(file, "File%d=%s\n",
-                      g_list_position(playlist, node) + 1, entry->filename);
-            continue;
-        }
+        PlaylistEntry *entry = PLAYLIST_ENTRY(node->data);
 
         if (entry->title && cfg.use_pl_metadata) {
             gint seconds;
@@ -1126,9 +1208,50 @@ playlist_save(const gchar * filename, gboolean is_pls)
 
             g_fprintf(file, "#EXTINF:%d,%s\n", seconds, entry->title);
         }
+
         g_fprintf(file, "%s\n", entry->filename);
     }
+
     PLAYLIST_UNLOCK();
+}
+
+static void
+playlist_save_pls(FILE * file)
+{
+    GList *node;
+
+    g_return_if_fail(file != NULL);
+
+    g_fprintf(file, "[playlist]\n");
+    g_fprintf(file, "NumberOfEntries=%d\n", playlist_get_length());
+
+    PLAYLIST_LOCK();
+    
+    for (node = playlist; node; node = g_list_next(node)) {
+        PlaylistEntry *entry = PLAYLIST_ENTRY(node->data);
+        
+        g_fprintf(file, "File%d=%s\n", g_list_position(playlist, node) + 1,
+                  entry->filename);
+    }
+    
+    PLAYLIST_UNLOCK();
+}
+
+gboolean
+playlist_save(const gchar * filename,
+              PlaylistFormat format)
+{
+    FILE *file;
+
+    g_return_val_if_fail(filename != NULL, FALSE);
+
+    playlist_set_current_name(filename);
+
+    if ((file = fopen(filename, "w")) == NULL)
+        return FALSE;
+
+    playlist_save_func_table[format](file);
+    
     return (fclose(file) == 0);
 }
 
@@ -1254,7 +1377,7 @@ playlist_load_m3u(const gchar * filename, gint pos)
     gchar *line;
     gchar *ext_info = NULL, *ext_title = NULL;
     gsize line_len = 1024;
-    gint ext_len;
+    gint ext_len = -1;
     gboolean is_extm3u = FALSE;
     guint added_count = 0;
 
@@ -1475,7 +1598,8 @@ playlist_get_songtime(guint pos)
 }
 
 static gint
-playlist_compare_title(PlaylistEntry * a, PlaylistEntry * b)
+playlist_compare_title(const PlaylistEntry * a,
+                       const PlaylistEntry * b)
 {
     const gchar *a_title, *b_title;
 
@@ -1504,7 +1628,8 @@ playlist_compare_title(PlaylistEntry * a, PlaylistEntry * b)
 }
 
 static gint
-playlist_compare_filename(PlaylistEntry * a, PlaylistEntry * b)
+playlist_compare_filename(const PlaylistEntry * a,
+                          const PlaylistEntry * b)
 {
     gchar *a_filename, *b_filename;
 
@@ -1553,13 +1678,15 @@ path_compare(const gchar * a, const gchar * b)
 }
 
 static gint
-playlist_compare_path(PlaylistEntry * a, PlaylistEntry * b)
+playlist_compare_path(const PlaylistEntry * a,
+                      const PlaylistEntry * b)
 {
     return path_compare(a->filename, b->filename);
 }
 
 static gint
-playlist_compare_date(PlaylistEntry * a, PlaylistEntry * b)
+playlist_compare_date(const PlaylistEntry * a,
+                      const PlaylistEntry * b)
 {
     struct stat buf;
     time_t modtime;
@@ -1855,13 +1982,17 @@ playlist_get_info_func(gpointer arg)
     while (playlist_get_info_is_going()) {
         PlaylistEntry *entry;
 
-        if (cfg.use_pl_metadata && cfg.get_info_on_load &&
+        if (cfg.use_pl_metadata &&
+            cfg.get_info_on_load &&
             playlist_get_info_scan_active) {
+
             PLAYLIST_LOCK();
             for (node = playlist_get(); node; node = g_list_next(node)) {
                 entry = node->data;
+
                 if (entry->title || entry->length != -1)
                     continue;
+
                 if (!playlist_entry_get_info(entry)) {
                     if (g_list_index(playlist_get(), entry) == -1)
                         /* Entry disappeared while we looked it up.
@@ -1880,9 +2011,12 @@ playlist_get_info_func(gpointer arg)
             if (!node)
                 playlist_get_info_scan_active = FALSE;
         }
-        else if (!cfg.get_info_on_load && cfg.get_info_on_demand &&
-                 cfg.playlist_visible && !cfg.playlist_shaded &&
+        else if (!cfg.get_info_on_load &&
+                 cfg.get_info_on_demand &&
+                 cfg.playlist_visible &&
+                 !cfg.playlist_shaded &&
                  cfg.use_pl_metadata) {
+
             gboolean found = FALSE;
 
             PLAYLIST_LOCK();
@@ -1974,7 +2108,6 @@ playlist_remove_dead_files(void)
 {
     GList *node, *next_node;
 
-
     PLAYLIST_LOCK();
 
     for (node = playlist; node; node = next_node) {
@@ -2000,7 +2133,10 @@ playlist_remove_dead_files(void)
             if (bmp_playback_get_playing())
                 continue;
 
-            playlist_position = PLAYLIST_ENTRY(next_node->data);
+            if (next_node)
+                playlist_position = PLAYLIST_ENTRY(next_node->data);
+            else
+                playlist_position = NULL;
         }
 
         playlist_entry_free(entry);
@@ -2018,8 +2154,10 @@ static gulong pl_total_time = 0, pl_selection_time = 0;
 static gboolean pl_total_more = FALSE, pl_selection_more = FALSE;
 
 void
-playlist_get_total_time(gulong * total_time, gulong * selection_time,
-                        gboolean * total_more, gboolean * selection_more)
+playlist_get_total_time(gulong * total_time,
+                        gulong * selection_time,
+                        gboolean * total_more,
+                        gboolean * selection_more)
 {
     PLAYLIST_LOCK();
     *total_time = pl_total_time;
@@ -2210,11 +2348,34 @@ playlist_set_shuffle(gboolean shuffle)
 }
 
 void
-playlist_new(void) {
-        playlist_set_current_name(NULL);
-        playlist_clear();
-        mainwin_clear_song_info();
-        mainwin_set_info_text();
+playlist_new(void) 
+{
+    playlist_set_current_name(NULL);
+    playlist_clear();
+    mainwin_clear_song_info();
+    mainwin_set_info_text();
 }
 
 
+const gchar *
+playlist_get_filename_to_play(void)
+{
+    const gchar *filename = NULL;
+    
+    PLAYLIST_LOCK();
+
+    if (playlist) {
+        if (!playlist_position) {
+            if (cfg.shuffle)
+                playlist_position = shuffle_list->data;
+            else
+                playlist_position = playlist->data;
+        }
+
+        filename = playlist_position->filename;
+    }
+
+    PLAYLIST_UNLOCK();
+
+    return filename;
+}
