@@ -81,30 +81,22 @@ enum {
     PLAYLISTWIN_SORT_RANDOMIZE, PLAYLISTWIN_SORT_REVERSE
 };
 
-enum {
-    PLAYLISTWIN_SAVE_EXTENSION = 0,
-    PLAYLISTWIN_SAVE_M3U,
-    PLAYLISTWIN_SAVE_PLS
-};
-
-
 GtkWidget *playlistwin;
 
-static gboolean playlistwin_resizing = FALSE;
-
 PlayList_List *playlistwin_list = NULL;
 PButton *playlistwin_shade, *playlistwin_close;
 Vis *playlistwin_vis;
 
+static gboolean playlistwin_resizing = FALSE;
+
 static GtkItemFactory *playlistwin_popup_menu;
-static GtkItemFactory *playlistwin_playlist_menu;
-static GtkItemFactory *plsort_menu, *playlistwin_sub_menu;
-static GtkItemFactory *pladd_menu, *pldel_menu, *plsel_menu, *pllist_menu;
+static GtkItemFactory *pladd_menu, *pldel_menu;
+static GtkItemFactory *plsel_menu, *plsort_menu;
+static GtkItemFactory *pllist_menu;
 
 static GdkPixmap *playlistwin_bg;
 static GdkBitmap *playlistwin_mask = NULL;
 static GdkGC *playlistwin_gc;
-static gint playlistwin_save_type;
 
 static GtkAccelGroup *playlistwin_accel;
 
@@ -126,11 +118,120 @@ static void plsort_menu_callback(gpointer cb_data, guint action,
                                            GtkWidget * w);
 static void playlistwin_sub_menu_callback(gpointer cb_data, guint action,
                                           GtkWidget * w);
-static void playlistwin_save_type_callback(gpointer cb_data, guint action,
-                                           GtkWidget * w);
 static void playlistwin_popup_menu_callback(gpointer cb_data, guint action,
                                             GtkWidget * w);
 
+static GtkItemFactoryEntry playlistwin_popup_menu_entries[] = {
+    {N_("/View Track Details"), NULL,
+     playlistwin_popup_menu_callback,
+     MISC_FILEINFO, "<ImageItem>", my_pixbuf},
+
+    ITEM_SEPARATOR,
+
+    {N_("/Remove Selected"), "Delete",
+     playlistwin_sub_menu_callback,
+     SUB_SELECTED, "<Item>", GTK_STOCK_DELETE},
+
+    {N_("/Remove Unselected"), NULL,
+     playlistwin_sub_menu_callback,
+     SUB_CROP, "<Item>", GTK_STOCK_CUT},
+
+    {N_("/Remove All"), NULL,
+     playlistwin_sub_menu_callback,
+     SUB_ALL, "<Item>", GTK_STOCK_DELETE},
+
+    ITEM_SEPARATOR,
+
+    {N_("/Queue Toggle"), "q",
+     playlistwin_popup_menu_callback,
+     MISC_QUEUE, "<Item>", NULL},
+};
+
+static GtkItemFactoryEntry pladd_menu_entries[] = {
+    {N_("/Add CD..."), "<shift>c",
+     mainwin_general_menu_callback,
+     MAINWIN_GENERAL_ADDCD, "<StockItem>", GTK_STOCK_CDROM},
+
+    {N_("/Add Internet Address..."), "<control>h",
+     mainwin_general_menu_callback,
+     MAINWIN_GENERAL_PLAYLOCATION, "<StockItem>", GTK_STOCK_NETWORK},
+
+    /* GtkFileChooser in SELECT_FOLDER mode is currently BROKEN! */
+#if 0
+    {N_("/Add Folders..."), "d",
+     mainwin_general_menu_callback,
+     MAINWIN_GENERAL_PLAYDIRECTORY, "<StockItem>", GTK_STOCK_OPEN},
+#endif
+
+    {N_("/Add Files..."), "f",
+     mainwin_general_menu_callback,
+     MAINWIN_GENERAL_PLAYFILE, "<StockItem>", GTK_STOCK_OPEN},
+};
+
+static GtkItemFactoryEntry pldel_menu_entries[] = {
+    {N_("/Clear Queue"), "<shift>Q",
+     playlistwin_popup_menu_callback,
+     PLIST_CQUEUE, "<Item>"},
+
+    ITEM_SEPARATOR,
+
+    {N_("/Remove Unavailable Files"), NULL,
+     playlistwin_sub_menu_callback,
+     PLAYLISTWIN_REMOVE_DEAD_FILES, "<Item>", GTK_STOCK_DELETE},
+
+    ITEM_SEPARATOR,
+
+    {N_("/Remove All"), NULL,
+     playlistwin_sub_menu_callback,
+     SUB_ALL, "<Item>", GTK_STOCK_DELETE},
+
+    {N_("/Remove Unselected"), NULL,
+     playlistwin_sub_menu_callback,
+     SUB_CROP, "<Item>", GTK_STOCK_DELETE},
+
+    {N_("/Remove Selected"), "Delete",
+     playlistwin_sub_menu_callback,
+     SUB_SELECTED, "<Item>", GTK_STOCK_DELETE}
+};
+
+static GtkItemFactoryEntry pllist_menu_entries[] = {
+    {N_("/New List"), NULL,
+     playlistwin_sub_menu_callback,
+     PLIST_NEW, "<StockItem>", GTK_STOCK_NEW},
+
+    ITEM_SEPARATOR,
+
+    {N_("/Load List"), "o",
+     playlistwin_sub_menu_callback,
+     PLIST_LOAD, "<StockItem>", GTK_STOCK_OPEN},
+
+    {N_("/Save List"), "s",
+     playlistwin_sub_menu_callback,
+     PLIST_SAVE, "<StockItem>", GTK_STOCK_SAVE},
+
+    ITEM_SEPARATOR,
+
+    {N_("/Update View"), "F5",
+     playlistwin_sub_menu_callback,
+     PLAYLISTWIN_REFRESH, "<StockItem>", GTK_STOCK_REFRESH}
+};
+
+static GtkItemFactoryEntry plsel_menu_entries[] = {
+    {N_("/Invert Selection"), NULL,
+     playlistwin_sub_menu_callback,
+     SEL_INV, "<Item>", GTK_STOCK_NETWORK},
+    
+    ITEM_SEPARATOR,
+
+    {N_("/Select None"),"<Ctrl><Shift>A",
+     playlistwin_sub_menu_callback,
+     SEL_ZERO, "<Item>", GTK_STOCK_OPEN},
+
+    {N_("/Select All"), "<Ctrl>A",
+     playlistwin_sub_menu_callback,
+     SEL_ALL, "<Item>", GTK_STOCK_OPEN},
+};
+
 static GtkItemFactoryEntry plsort_menu_entries[] = {
     {N_("/Randomize List"), NULL, plsort_menu_callback,
      PLAYLISTWIN_SORT_RANDOMIZE, "<Item>"},
@@ -157,136 +258,6 @@ static GtkItemFactoryEntry plsort_menu_entries[] = {
      PLAYLISTWIN_SORT_SEL_BYDATE, "<Item>"}
 };
 
-static GtkItemFactoryEntry playlistwin_playlist_filetypes[] = {
-    {N_("/By extension"), NULL, playlistwin_save_type_callback,
-     PLAYLISTWIN_SAVE_EXTENSION, NULL},
-    ITEM_SEPARATOR,
-    {"/m3u", NULL, playlistwin_save_type_callback, PLAYLISTWIN_SAVE_M3U,
-     NULL},
-    {"/pls", NULL, playlistwin_save_type_callback, PLAYLISTWIN_SAVE_PLS,
-     NULL}
-};
-
-static const gint playlistwin_playlist_filetypes_num =
-    G_N_ELEMENTS(playlistwin_playlist_filetypes);
-
-
-static GtkItemFactoryEntry playlistwin_sub_menu_entries[] = {
-    {N_("/Remove Unavailable Files"), NULL, playlistwin_sub_menu_callback,
-     PLAYLISTWIN_REMOVE_DEAD_FILES, "<Item>"},
-};
-
-static const gint playlistwin_sub_menu_entries_num =
-    G_N_ELEMENTS(playlistwin_sub_menu_entries);
-
-static GtkItemFactoryEntry playlistwin_popup_menu_entries[] = {
-    {N_("/View Track Details"), NULL, playlistwin_popup_menu_callback,
-     MISC_FILEINFO, "<ImageItem>", my_pixbuf},
-    ITEM_SEPARATOR,
-    {N_("/Remove Selected"), "Delete", playlistwin_sub_menu_callback,
-     SUB_SELECTED, "<Item>", GTK_STOCK_DELETE},
-    {N_("/Remove Unselected"), NULL, playlistwin_sub_menu_callback, SUB_CROP,
-     "<Item>", GTK_STOCK_CUT},
-    {N_("/Remove All"), NULL,
-     playlistwin_sub_menu_callback,
-     SUB_ALL, "<Item>", GTK_STOCK_DELETE},
-    {"/-", NULL, NULL, 0, "<Separator>"},
-    {N_("/Queue Toggle"), "q", playlistwin_popup_menu_callback, MISC_QUEUE,
-     "<Item>", NULL},
-};
-
-static const gint playlistwin_popup_menu_entries_num =
-    G_N_ELEMENTS(playlistwin_popup_menu_entries);
-
-static GtkItemFactoryEntry playlist_menu_entries[] = {
-    {N_("/New List"), "n", playlistwin_sub_menu_callback, PLIST_NEW,
-     "<StockItem>", GTK_STOCK_NEW},
-    {"/-", NULL, NULL, 0, "<Separator>"},
-    {N_("/Load List"), "o", playlistwin_sub_menu_callback,
-     PLIST_LOAD, "<StockItem>", GTK_STOCK_OPEN},
-    {N_("/Save List"), "s", playlistwin_sub_menu_callback,
-     PLIST_SAVE, "<StockItem>", GTK_STOCK_SAVE},
-    {"/-", NULL, NULL, 0, "<Separator>"},
-    {N_("/Update View"), "F5", playlistwin_sub_menu_callback,
-     PLAYLISTWIN_REFRESH, "<StockItem>", GTK_STOCK_REFRESH}
-};
-
-static const gint playlist_menu_entries_num =
-    G_N_ELEMENTS(playlist_menu_entries);
-
-
-static GtkItemFactoryEntry pladd_menu_entries[] = {
-    {N_("/Add CD..."), "<shift>c", mainwin_general_menu_callback,
-     MAINWIN_GENERAL_ADDCD, "<StockItem>", GTK_STOCK_CDROM},
-    {N_("/Add Internet Address..."), "<control>h",
-     mainwin_general_menu_callback,
-     MAINWIN_GENERAL_PLAYLOCATION, "<StockItem>", GTK_STOCK_NETWORK},
-
-    /* GtkFileChooser in SELECT_FOLDER mode is currently BROKEN! */
-/*
-    {N_("/Add Folders..."), "d",
-     mainwin_general_menu_callback,
-     MAINWIN_GENERAL_PLAYDIRECTORY, "<StockItem>", GTK_STOCK_OPEN},
-*/
-    {N_("/Add Files..."), "f", mainwin_general_menu_callback,
-     MAINWIN_GENERAL_PLAYFILE, "<StockItem>", GTK_STOCK_OPEN},
-};
-
-static GtkItemFactoryEntry pldel_menu_entries[] = {
-    {N_("/Clear Queue"), "<shift>Q", playlistwin_popup_menu_callback,
-     PLIST_CQUEUE, "<Item>"},
-
-    ITEM_SEPARATOR,
-
-    {N_("/Remove Selected"), "Delete", playlistwin_sub_menu_callback,
-     SUB_SELECTED, "<Item>", GTK_STOCK_DELETE},
-
-    {N_("/Remove Unselected"), NULL, playlistwin_sub_menu_callback,
-     SUB_CROP, "<Item>", GTK_STOCK_DELETE},
-
-    {N_("/Remove All"), NULL, playlistwin_sub_menu_callback,
-     SUB_ALL, "<Item>", GTK_STOCK_DELETE},
-
-    ITEM_SEPARATOR,
-
-    {N_("/Remove Unavailable Files"), NULL, playlistwin_sub_menu_callback,
-     PLAYLISTWIN_REMOVE_DEAD_FILES, "<Item>", GTK_STOCK_DELETE},
-
-};
-
-static GtkItemFactoryEntry plsel_menu_entries[] = {
-
-    {N_("/Invert Selection"), NULL,
-     playlistwin_sub_menu_callback,
-     SEL_INV, "<Item>", GTK_STOCK_NETWORK},
-    
-    ITEM_SEPARATOR,
-
-    {N_("/Select None"),"<Ctrl><Shift>A",
-     playlistwin_sub_menu_callback,
-     SEL_ZERO, "<Item>", GTK_STOCK_OPEN},
-
-    {N_("/Select All"), "<Ctrl>A", playlistwin_sub_menu_callback,
-     SEL_ALL, "<Item>", GTK_STOCK_OPEN},
-};
-
-
-static GtkItemFactoryEntry pllist_menu_entries[] = {
-
-    {N_("/New List"), NULL,
-     playlistwin_sub_menu_callback,
-     PLIST_NEW, "<StockItem>", GTK_STOCK_NEW},
-    
-    ITEM_SEPARATOR,
-
-    {N_("/Save List"),NULL,
-     playlistwin_sub_menu_callback,
-     PLIST_SAVE, "<Item>", GTK_STOCK_OPEN},
-
-    {N_("/Load List"), NULL, playlistwin_sub_menu_callback,
-     PLIST_LOAD, "<Item>", GTK_STOCK_OPEN},
-};
-
 
 static void playlistwin_draw_frame(void);
 
@@ -457,8 +428,8 @@ playlistwin_update_list(void)
 {
     g_return_if_fail(playlistwin_list != NULL);
 
-    draw_widget(playlistwin_list);
-    draw_widget(playlistwin_slider);
+    widget_draw(WIDGET(playlistwin_list));
+    widget_draw(WIDGET(playlistwin_slider));
     playlistwin_update_info();
     playlistwin_update_sinfo();
     /*     mainwin_update_jtf(); */
@@ -537,7 +508,7 @@ playlistwin_set_shade(gboolean shaded)
     cfg.playlist_shaded = shaded;
 
     if (shaded) {
-        show_widget(playlistwin_sinfo);
+        widget_show(WIDGET(playlistwin_sinfo));
         playlistwin_shade->pb_nx = 128;
         playlistwin_shade->pb_ny = 45;
         playlistwin_shade->pb_px = 150;
@@ -546,7 +517,7 @@ playlistwin_set_shade(gboolean shaded)
         playlistwin_close->pb_ny = 45;
     }
     else {
-        hide_widget(playlistwin_sinfo);
+        widget_hide(WIDGET(playlistwin_sinfo));
         playlistwin_shade->pb_nx = 157;
         playlistwin_shade->pb_ny = 3;
         playlistwin_shade->pb_px = 62;
@@ -677,49 +648,50 @@ playlistwin_resize(gint width, gint height)
     /* FIXME: why the fsck are we doing this manually? */
     /* adjust widget positions and sizes */
 
-    resize_widget(playlistwin_list, width - 31, height - 58);
+    widget_resize(WIDGET(playlistwin_list), width - 31, height - 58);
 
-    move_widget(playlistwin_slider, width - 15, 20);
-    resize_widget(playlistwin_slider, 8, height - 58);
+    widget_move(WIDGET(playlistwin_slider), width - 15, 20);
+    widget_resize(WIDGET(playlistwin_slider), 8, height - 58);
 
-    resize_widget(playlistwin_sinfo, width - 35, 14);
+    widget_resize(WIDGET(playlistwin_sinfo), width - 35, 14);
     playlistwin_update_sinfo();
 
-    move_widget(playlistwin_shade, width - 21, 3);
-    move_widget(playlistwin_close, width - 11, 3);
-    move_widget(playlistwin_time_min, width - 82, height - 15);
-    move_widget(playlistwin_time_sec, width - 64, height - 15);
-    move_widget(playlistwin_info, width - 143, height - 28);
-    move_widget(playlistwin_srew, width - 144, height - 16);
-    move_widget(playlistwin_splay, width - 138, height - 16);
-    move_widget(playlistwin_spause, width - 128, height - 16);
-    move_widget(playlistwin_sstop, width - 118, height - 16);
-    move_widget(playlistwin_sfwd, width - 109, height - 16);
-    move_widget(playlistwin_seject, width - 100, height - 16);
-    move_widget(playlistwin_sscroll_up, width - 14, height - 35);
-    move_widget(playlistwin_sscroll_down, width - 14, height - 30);
+    widget_move(WIDGET(playlistwin_shade), width - 21, 3);
+    widget_move(WIDGET(playlistwin_close), width - 11, 3);
+    widget_move(WIDGET(playlistwin_time_min), width - 82, height - 15);
+    widget_move(WIDGET(playlistwin_time_sec), width - 64, height - 15);
+    widget_move(WIDGET(playlistwin_info), width - 143, height - 28);
+    widget_move(WIDGET(playlistwin_srew), width - 144, height - 16);
+    widget_move(WIDGET(playlistwin_splay), width - 138, height - 16);
+    widget_move(WIDGET(playlistwin_spause), width - 128, height - 16);
+    widget_move(WIDGET(playlistwin_sstop), width - 118, height - 16);
+    widget_move(WIDGET(playlistwin_sfwd), width - 109, height - 16);
+    widget_move(WIDGET(playlistwin_seject), width - 100, height - 16);
+    widget_move(WIDGET(playlistwin_sscroll_up), width - 14, height - 35);
+    widget_move(WIDGET(playlistwin_sscroll_down), width - 14, height - 30);
 
     /* decide if we should show the mini visualizer */
     if (playlistwin_get_width() >= 350) {
-        move_widget(playlistwin_vis, width - 223, height - 26);
+        widget_move(WIDGET(playlistwin_vis), width - 223, height - 26);
+
         if (playlistwin_vis_enabled)
-            show_widget(playlistwin_vis);
+            widget_show(WIDGET(playlistwin_vis));
     }
     else
-        hide_widget(playlistwin_vis);
+        widget_hide(WIDGET(playlistwin_vis));
 
     g_object_unref(playlistwin_bg);
     playlistwin_bg = gdk_pixmap_new(playlistwin->window, width, height, -1);
     playlistwin_set_mask();
 
-    lock_widget_list(playlistwin_wlist);
+    widget_list_lock(playlistwin_wlist);
 
     widget_list_change_pixmap(playlistwin_wlist, playlistwin_bg);
     playlistwin_draw_frame();
-    draw_widget_list(playlistwin_wlist, &redraw, TRUE);
-    clear_widget_list_redraw(playlistwin_wlist);
+    widget_list_draw(playlistwin_wlist, &redraw, TRUE);
+    widget_list_clear_redraw(playlistwin_wlist);
 
-    unlock_widget_list(playlistwin_wlist);
+    widget_list_unlock(playlistwin_wlist);
 
     gdk_window_set_back_pixmap(playlistwin->window, playlistwin_bg, 0);
     gdk_window_clear(playlistwin->window);
@@ -796,28 +768,35 @@ playlistwin_set_sensitive_sortmenu(void)
 }
 
 static void
-playlistwin_save_playlist_error(const gchar * path, GtkWindow * file_selector)
+show_playlist_save_error(GtkWindow * parent,
+                         const gchar * filename)
 {
     GtkWidget *dialog;
-    gint result;
+    
+    g_return_if_fail(GTK_IS_WINDOW(parent));
+    g_return_if_fail(filename != NULL);
 
-    dialog = gtk_message_dialog_new(GTK_WINDOW(file_selector),
+    dialog = gtk_message_dialog_new(GTK_WINDOW(parent),
                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_MESSAGE_ERROR,
                                     GTK_BUTTONS_OK,
                                     _("Error writing playlist \"%s\": %s"),
-                                    path, strerror(errno));
+                                    filename, strerror(errno));
 
-    result = gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_dialog_run(GTK_DIALOG(dialog));
     gtk_widget_destroy(dialog);
 }
 
 static gboolean
-check_overwrite_dialog(GtkWindow * parent, const gchar * filename)
+show_playlist_overwrite_prompt(GtkWindow * parent,
+                               const gchar * filename)
 {
     GtkWidget *dialog;
     gint result;
 
+    g_return_val_if_fail(GTK_IS_WINDOW(parent), FALSE);
+    g_return_val_if_fail(filename != NULL, FALSE);
+
     dialog = gtk_message_dialog_new(GTK_WINDOW(parent),
                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_MESSAGE_QUESTION,
@@ -831,45 +810,49 @@ check_overwrite_dialog(GtkWindow * parent, const gchar * filename)
     return (result == GTK_RESPONSE_YES);
 }
 
+static void
+show_playlist_save_format_error(GtkWindow * parent,
+                                const gchar * filename)
+{
+    const gchar *markup = 
+        N_("<b><big>Unable to save playlist.</big></b>\n\n"
+           "Unknown file type for '%s'.\n");
+
+    GtkWidget *dialog;
+
+    g_return_if_fail(GTK_IS_WINDOW(parent));
+    g_return_if_fail(filename != NULL);
+
+    dialog =
+        gtk_message_dialog_new_with_markup(GTK_WINDOW(parent),
+                                           GTK_DIALOG_DESTROY_WITH_PARENT,
+                                           GTK_MESSAGE_ERROR,
+                                           GTK_BUTTONS_OK,
+                                           _(markup),
+                                           filename);
+    gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
+}
+
 static void
 playlistwin_save_playlist(const gchar * filename)
 {
-    gboolean is_pls = FALSE;
+    PlaylistFormat format;
 
-    if (playlistwin_save_type == PLAYLISTWIN_SAVE_EXTENSION) {
-        gboolean error = FALSE;
-
-        if (!is_playlist_name(filename))
-            error = TRUE;
-
-        if (g_str_has_suffix(filename, ".pls"))
-            is_pls = TRUE;
-
-        if (error) {
-            GtkWidget *dialog;
-            dialog = gtk_message_dialog_new(GTK_WINDOW(playlistwin),
-                                            GTK_DIALOG_DESTROY_WITH_PARENT,
-                                            GTK_MESSAGE_ERROR,
-                                            GTK_BUTTONS_OK,
-                                            _("Unable to save playlist! "
-                                              "Unknown file type for %s"),
-                                            filename);
-            gtk_dialog_run(GTK_DIALOG(dialog));
-            gtk_widget_destroy(dialog);
-            return;
-        }
+    format = playlist_format_get_from_name(filename);
+    if (format == PLAYLIST_FORMAT_UNKNOWN) {
+        show_playlist_save_format_error(GTK_WINDOW(playlistwin), filename);
+        return;
     }
-    else if (playlistwin_save_type == PLAYLISTWIN_SAVE_PLS)
-        is_pls = TRUE;
 
     str_replace_in(&cfg.playlist_path, g_path_get_dirname(filename));
 
     if (g_file_test(filename, G_FILE_TEST_IS_REGULAR))
-        if (!check_overwrite_dialog(GTK_WINDOW(playlistwin), filename))
+        if (!show_playlist_overwrite_prompt(GTK_WINDOW(playlistwin), filename))
             return;
 
-    if (!playlist_save(filename, is_pls))
-        playlistwin_save_playlist_error(filename, GTK_WINDOW(playlistwin));
+    if (!playlist_save(filename, format))
+        show_playlist_save_error(GTK_WINDOW(playlistwin), filename);
 }
 
 #if 0
@@ -917,7 +900,7 @@ playlist_file_selection(const gchar * title,
         gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog),
                                       default_filename);
 
-    button = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CLOSE,
+    button = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL,
                                    GTK_RESPONSE_REJECT);
     gtk_button_set_use_stock(GTK_BUTTON(button), TRUE);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
@@ -926,6 +909,7 @@ playlist_file_selection(const gchar * title,
                                    save ? GTK_STOCK_SAVE : GTK_STOCK_OPEN,
                                    GTK_RESPONSE_ACCEPT);
     gtk_button_set_use_stock(GTK_BUTTON(button), TRUE);
+    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
 
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
         filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
@@ -941,7 +925,8 @@ void
 playlistwin_select_playlist_to_load(const gchar * default_filename)
 {
     gchar *filename =
-        playlist_file_selection(_("Load playlist"), FALSE, default_filename);
+        playlist_file_selection(_("Load Playlist"), FALSE, default_filename);
+
     if (filename) {
         playlistwin_load_playlist(filename);
         g_free(filename);
@@ -952,19 +937,24 @@ static void
 playlistwin_select_playlist_to_save(const gchar * default_filename)
 {
     gchar *filename =
-        playlist_file_selection(_("Save playlist"), TRUE, default_filename);
+        playlist_file_selection(_("Save Playlist"), TRUE, default_filename);
+
     if (filename) {
+        /* Default to M3U if no filename has extension */
+
+        /* NOTE: This doesn't work correctly for hidden files
+           - descender */
+        if (!strchr(filename, '.')) {
+            gchar *tmpstr = filename;
+            filename = g_strconcat(filename, ".m3u", NULL);
+            g_free(tmpstr);
+        }
+
         playlistwin_save_playlist(filename);
         g_free(filename);
     }
 }
 
-static void
-playlistwin_save_type_callback(gpointer cb_data, guint action, GtkWidget * w)
-{
-    playlistwin_save_type = action;
-}
-
 static void
 playlistwin_run_dirbrowser(void)
 {
@@ -974,22 +964,22 @@ playlistwin_run_dirbrowser(void)
 static gboolean
 inside_sensitive_widgets(gint x, gint y)
 {
-    return (inside_widget(x, y, playlistwin_list) ||
-            inside_widget(x, y, playlistwin_slider) ||
-            inside_widget(x, y, playlistwin_close) ||
-            inside_widget(x, y, playlistwin_shade) ||
-            inside_widget(x, y, playlistwin_time_min) ||
-            inside_widget(x, y, playlistwin_time_sec) ||
-            inside_widget(x, y, playlistwin_info) ||
-            inside_widget(x, y, playlistwin_vis) ||
-            inside_widget(x, y, playlistwin_srew) ||
-            inside_widget(x, y, playlistwin_splay) ||
-            inside_widget(x, y, playlistwin_spause) ||
-            inside_widget(x, y, playlistwin_sstop) ||
-            inside_widget(x, y, playlistwin_sfwd) ||
-            inside_widget(x, y, playlistwin_seject) ||
-            inside_widget(x, y, playlistwin_sscroll_up) ||
-            inside_widget(x, y, playlistwin_sscroll_down));
+    return (widget_contains(WIDGET(playlistwin_list), x, y) ||
+            widget_contains(WIDGET(playlistwin_slider), x, y) ||
+            widget_contains(WIDGET(playlistwin_close), x, y) ||
+            widget_contains(WIDGET(playlistwin_shade), x, y) ||
+            widget_contains(WIDGET(playlistwin_time_min), x, y) ||
+            widget_contains(WIDGET(playlistwin_time_sec), x, y) ||
+            widget_contains(WIDGET(playlistwin_info), x, y) ||
+            widget_contains(WIDGET(playlistwin_vis), x, y) ||
+            widget_contains(WIDGET(playlistwin_srew), x, y) ||
+            widget_contains(WIDGET(playlistwin_splay), x, y) ||
+            widget_contains(WIDGET(playlistwin_spause), x, y) ||
+            widget_contains(WIDGET(playlistwin_sstop), x, y) ||
+            widget_contains(WIDGET(playlistwin_sfwd), x, y) ||
+            widget_contains(WIDGET(playlistwin_seject), x, y) ||
+            widget_contains(WIDGET(playlistwin_sscroll_up), x, y) ||
+            widget_contains(WIDGET(playlistwin_sscroll_down), x, y));
 }
 
 #define REGION_L(x1,x2,y1,y2)                   \
@@ -1039,13 +1029,18 @@ playlistwin_press(GtkWidget * widget,
          (cfg.playlist_shaded &&
           event->x >= playlistwin_get_width() - 31 &&
           event->x < playlistwin_get_width() - 22))) {
-        /* resize area */
-        playlistwin_resizing = TRUE;
-        gtk_window_begin_resize_drag(GTK_WINDOW(widget),
-                                     GDK_WINDOW_EDGE_SOUTH_EAST,
-                                     event->button,
-                                     event->x + xpos, event->y + ypos,
-                                     event->time);
+
+        /* NOTE: Workaround for bug #214 */
+        if (event->type != GDK_2BUTTON_PRESS && 
+            event->type != GDK_3BUTTON_PRESS) {
+            /* resize area */
+            playlistwin_resizing = TRUE;
+            gtk_window_begin_resize_drag(GTK_WINDOW(widget),
+                                         GDK_WINDOW_EDGE_SOUTH_EAST,
+                                         event->button,
+                                         event->x + xpos, event->y + ypos,
+                                         event->time);
+        }
         grab = FALSE;
     }
     else if (event->button == 1 && REGION_L(12, 37, 29, 11)) {
@@ -1096,10 +1091,10 @@ playlistwin_press(GtkWidget * widget,
     }
     else if (event->button == 1 && REGION_R(46, 23, 29, 11)) {
         /* LIST button menu */
-        _menu = GTK_WIDGET(playlistwin_playlist_menu->widget);
+        _menu = GTK_WIDGET(pllist_menu->widget);
         if (!GTK_WIDGET_REALIZED(_menu)) gtk_widget_realize(_menu);
         gtk_widget_size_request(_menu, &req);
-        gtk_item_factory_popup_with_data(playlistwin_playlist_menu,
+        gtk_item_factory_popup_with_data(pllist_menu,
                                          NULL, NULL,
                                          xpos + playlistwin_get_width() - req.width - 12,
                                          (ypos + playlistwin_get_height()) - 8 - req.height, 1, event->time);
@@ -1112,7 +1107,7 @@ playlistwin_press(GtkWidget * widget,
             cfg.timer_mode = TIMER_ELAPSED;
     }
     else if (event->button == 2 && (event->type == GDK_BUTTON_PRESS) &&
-             inside_widget(event->x, event->y, playlistwin_list)) {
+             widget_contains(WIDGET(playlistwin_list), event->x, event->y)) {
         gtk_selection_convert(widget, GDK_SELECTION_PRIMARY,
                               GDK_TARGET_STRING, event->time);
     }
@@ -1148,7 +1143,7 @@ playlistwin_press(GtkWidget * widget,
         return TRUE;
     }
     else if (event->button == 3 &&
-             !(inside_widget(event->x, event->y, playlistwin_list) ||
+             !(widget_contains(WIDGET(playlistwin_list), event->x, event->y) ||
                (event->y >= cfg.playlist_height - 29 &&
                 event->y < cfg.playlist_height - 11 &&
                 ((event->x >= 12 && event->x < 37) ||
@@ -1166,7 +1161,7 @@ playlistwin_press(GtkWidget * widget,
         grab = FALSE;
     }
     else if (event->button == 3 &&
-             inside_widget(event->x, event->y, playlistwin_list)) {
+             widget_contains(WIDGET(playlistwin_list), event->x, event->y)) {
         /* popup menu */
         playlistwin_set_sensitive_sortmenu();
         gtk_item_factory_popup(playlistwin_popup_menu,
@@ -1406,8 +1401,8 @@ draw_playlist_window(gboolean force)
     if (force)
         playlistwin_draw_frame();
 
-    lock_widget_list(playlistwin_wlist);
-    draw_widget_list(playlistwin_wlist, &redraw, force);
+    widget_list_lock(playlistwin_wlist);
+    widget_list_draw(playlistwin_wlist, &redraw, force);
 
     if (redraw || force) {
         if (force) {
@@ -1427,7 +1422,7 @@ draw_playlist_window(gboolean force)
         gdk_flush();
     }
 
-    unlock_widget_list(playlistwin_wlist);
+    widget_list_unlock(playlistwin_wlist);
 }
 
 
@@ -1442,15 +1437,16 @@ void
 playlistwin_vis_enable(void)
 {
     playlistwin_vis_enabled = TRUE;
+
     if (playlistwin_get_width() >= 350)
-        show_widget(playlistwin_vis);
+        widget_show(WIDGET(playlistwin_vis));
 }
 
 void
 playlistwin_vis_disable(void)
 {
     playlistwin_vis_enabled = FALSE;
-    hide_widget(playlistwin_vis);
+    widget_hide(WIDGET(playlistwin_vis));
     draw_playlist_window(TRUE);
 }
 
@@ -1522,7 +1518,7 @@ playlistwin_drag_data_received(GtkWidget * widget,
         return;
     }
 
-    if (inside_widget(x, y, playlistwin_list)) {
+    if (widget_contains(WIDGET(playlistwin_list), x, y)) {
         pos = (y - WIDGET(playlistwin_list)->y) /
             playlistwin_list->pl_fheight + playlistwin_list->pl_first;
 
@@ -1544,7 +1540,7 @@ playlistwin_create_widgets(void)
                        4, 4, playlistwin_get_width() - 35, FALSE, SKIN_TEXT);
 
     if (!cfg.playlist_shaded)
-        hide_widget(playlistwin_sinfo);
+        widget_hide(WIDGET(playlistwin_sinfo));
 
     /* shade/unshade window push button */
     if (cfg.playlist_shaded)
@@ -1607,7 +1603,7 @@ playlistwin_create_widgets(void)
         create_vis(&playlistwin_wlist, playlistwin_bg, playlistwin->window,
                    playlistwin_gc, playlistwin_get_width() - 223,
                    cfg.playlist_height - 26, 72);
-    hide_widget(playlistwin_vis);
+    widget_hide(WIDGET(playlistwin_vis));
 
     /* mini play control buttons at right bottom corner */
 
@@ -1757,32 +1753,20 @@ playlistwin_create_popup_menus(void)
 {
     playlistwin_accel = gtk_accel_group_new();
 
-    /* playlist window sort menu */
-    plsort_menu = create_menu(plsort_menu_entries,
-                              G_N_ELEMENTS(plsort_menu_entries),
-                              playlistwin_accel);
-
-    /* playlist window sub menu */
-    playlistwin_sub_menu = create_menu(playlistwin_sub_menu_entries,
-                                       playlistwin_sub_menu_entries_num,
-                                       playlistwin_accel);
-
     /* playlist window popup menu */
     playlistwin_popup_menu = create_menu(playlistwin_popup_menu_entries,
                                          G_N_ELEMENTS(playlistwin_popup_menu_entries),
                                          playlistwin_accel);
 
-    /* playlist menu */
-    playlistwin_playlist_menu = create_menu(playlist_menu_entries,
-                                            playlist_menu_entries_num,
-                                            playlistwin_accel);
-
     pladd_menu  = create_menu(pladd_menu_entries, G_N_ELEMENTS(pladd_menu_entries),
                               playlistwin_accel);
     pldel_menu  = create_menu(pldel_menu_entries, G_N_ELEMENTS(pldel_menu_entries),
                               playlistwin_accel);
     plsel_menu  = create_menu(plsel_menu_entries, G_N_ELEMENTS(plsel_menu_entries),
                               playlistwin_accel);
+    plsort_menu = create_menu(plsort_menu_entries,
+                              G_N_ELEMENTS(plsort_menu_entries),
+                              playlistwin_accel);
     pllist_menu = create_menu(pllist_menu_entries, G_N_ELEMENTS(pllist_menu_entries),
                               playlistwin_accel);
 
