@@ -92,23 +92,23 @@ plugin_is_duplicate(const gchar * filename)
 
     /* FIXME: messy stuff */
 
-    for (l = ip_data->input_list; l; l = g_list_next(l))
+    for (l = ip_data.input_list; l; l = g_list_next(l))
         if (!strcmp(basename, g_basename(INPUT_PLUGIN(l->data)->filename)))
             return TRUE;
 
-    for (l = op_data->output_list; l; l = g_list_next(l))
+    for (l = op_data.output_list; l; l = g_list_next(l))
         if (!strcmp(basename, g_basename(OUTPUT_PLUGIN(l->data)->filename)))
             return TRUE;
 
-    for (l = ep_data->effect_list; l; l = g_list_next(l))
+    for (l = ep_data.effect_list; l; l = g_list_next(l))
         if (!strcmp(basename, g_basename(EFFECT_PLUGIN(l->data)->filename)))
             return TRUE;
 
-    for (l = gp_data->general_list; l; l = g_list_next(l))
+    for (l = gp_data.general_list; l; l = g_list_next(l))
         if (!strcmp(basename, g_basename(GENERAL_PLUGIN(l->data)->filename)))
             return TRUE;
 
-    for (l = vp_data->vis_list; l; l = g_list_next(l))
+    for (l = vp_data.vis_list; l; l = g_list_next(l))
         if (!strcmp(basename, g_basename(VIS_PLUGIN(l->data)->filename)))
             return TRUE;
 
@@ -117,11 +117,81 @@ plugin_is_duplicate(const gchar * filename)
 
 
 #define PLUGIN_GET_INFO(x) ((PluginGetInfoFunc)(x))()
-typedef gpointer (*PluginGetInfoFunc) (void);
+typedef Plugin * (*PluginGetInfoFunc) (void);
+
+static void
+input_plugin_init(Plugin * plugin)
+{
+    InputPlugin *p = INPUT_PLUGIN(plugin);
+
+    p->get_vis_type = input_get_vis_type;
+    p->add_vis_pcm = input_add_vis_pcm;
+    
+    /* Pretty const casts courtesy of XMMS's plugin.h legacy. Anyone
+       else thinks we could use a CONST macro to solve the warnings?
+       - descender */
+    p->set_info = (void (*)(gchar *, gint, gint, gint, gint)) playlist_set_info;
+    p->set_info_text = (void (*)(gchar *)) input_set_info_text;
+    
+    ip_data.input_list = g_list_append(ip_data.input_list, p);
+    
+    g_hash_table_replace(plugin_matrix, g_path_get_basename(p->filename),
+                         GINT_TO_POINTER(1));
+}
+
+static void
+output_plugin_init(Plugin * plugin)
+{
+    OutputPlugin *p = OUTPUT_PLUGIN(plugin);
+    op_data.output_list = g_list_append(op_data.output_list, p);    
+}
+
+static void
+effect_plugin_init(Plugin * plugin)
+{
+    EffectPlugin *p = EFFECT_PLUGIN(plugin);
+    ep_data.effect_list = g_list_append(ep_data.effect_list, p);
+}
+
+static void
+general_plugin_init(Plugin * plugin)
+{
+    GeneralPlugin *p = GENERAL_PLUGIN(plugin);
+    p->xmms_session = ctrlsocket_get_session_id();
+    gp_data.general_list = g_list_append(gp_data.general_list, p);
+}
+
+static void
+vis_plugin_init(Plugin * plugin)
+{
+    VisPlugin *p = VIS_PLUGIN(plugin);
+    p->xmms_session = ctrlsocket_get_session_id();
+    p->disable_plugin = vis_disable_plugin;
+    vp_data.vis_list = g_list_append(vp_data.vis_list, p);
+}
+
+
+/* FIXME: Placed here (hopefully) temporarily - descender */
+
+typedef struct {
+    const gchar *name;
+    const gchar *id;
+    void (*init)(Plugin *);
+} PluginType;
+
+static PluginType plugin_types[] = {
+    { "input"        , "get_iplugin_info", input_plugin_init },
+    { "output"       , "get_oplugin_info", output_plugin_init },
+    { "effect"       , "get_eplugin_info", effect_plugin_init },
+    { "general"      , "get_gplugin_info", general_plugin_init },
+    { "visualization", "get_vplugin_info", vis_plugin_init },
+    { NULL, NULL, NULL }
+};
 
 static void
 add_plugin(const gchar * filename)
 {
+    PluginType *type;
     GModule *module;
     gpointer func;
 
@@ -129,85 +199,23 @@ add_plugin(const gchar * filename)
         return;
 
     if (!(module = g_module_open(filename, 0))) {
-        g_warning("Failed to load plugin (%s)", g_module_error());
+        g_warning("Failed to load plugin (%s): %s", 
+                  filename, g_module_error());
         return;
     }
 
-    /* Input plugin */
-    if (g_module_symbol(module, "get_iplugin_info", &func)) {
+    for (type = plugin_types; type->name; type++)
+    {
+        if (g_module_symbol(module, type->id, &func)) {
+            Plugin *plugin = PLUGIN_GET_INFO(func);
 
-        InputPlugin *p = PLUGIN_GET_INFO(func);
-        p->handle = module;
-        p->filename = g_strdup(filename);
-        p->get_vis_type = input_get_vis_type;
-        p->add_vis_pcm = input_add_vis_pcm;
+            plugin->handle = module;
+            plugin->filename = g_strdup(filename);
+            type->init(PLUGIN_GET_INFO(func));
 
-        /* Pretty const casts courtesy of XMMS's plugin.h legacy. Anyone
-           else thinks we could use a CONST macro to solve the warnings?
-           - Descender */
-        p->set_info = (void (*)(gchar *, gint, gint, gint, gint)) playlist_set_info;
-        p->set_info_text = (void (*)(gchar *)) input_set_info_text;
-
-        ip_data->input_list = g_list_append(ip_data->input_list, p);
-
-        g_hash_table_replace(plugin_matrix, g_path_get_basename(filename),
-                             GINT_TO_POINTER(1));
-
-        g_message("Loaded input plugin (%s)", p->filename);
-
-        return;
-    }
-
-    /* Output plugin */
-    if (g_module_symbol(module, "get_oplugin_info", &func)) {
-
-        OutputPlugin *p = PLUGIN_GET_INFO(func);
-        p->handle = module;
-        p->filename = g_strdup(filename);
-        op_data->output_list = g_list_append(op_data->output_list, p);
-        g_message("Loaded output plugin (%s)", filename);
-
-        return;
-    }
-
-    /* Effects plugin */
-    if (g_module_symbol(module, "get_eplugin_info", &func)) {
-
-        EffectPlugin *p = PLUGIN_GET_INFO(func);
-        p->handle = module;
-        p->filename = g_strdup(filename);
-        ep_data->effect_list = g_list_append(ep_data->effect_list, p);
-        g_message("Loaded effect plugin (%s)", filename);
-
-        return;
-    }
-
-    /* General plugin */
-    if (g_module_symbol(module, "get_gplugin_info", &func)) {
-
-        GeneralPlugin *p = PLUGIN_GET_INFO(func);
-        p->handle = module;
-        p->filename = g_strdup(filename);
-        p->xmms_session = ctrlsocket_get_session_id();
-        gp_data->general_list = g_list_append(gp_data->general_list, p);
-        g_message("Loaded general plugin (%s)", filename);
-
-        return;
-    }
-
-
-    /* Vis plugin */
-    if (g_module_symbol(module, "get_vplugin_info", &func)) {
-
-        VisPlugin *p = PLUGIN_GET_INFO(func);
-        p->handle = module;
-        p->filename = g_strdup(filename);
-        p->xmms_session = ctrlsocket_get_session_id();
-        p->disable_plugin = vis_disable_plugin;
-        vp_data->vis_list = g_list_append(vp_data->vis_list, p);
-        g_message("Loaded visualization plugin (%s)", filename);
-
-        return;
+            g_message("Loaded %s plugin (%s)", type->name, filename);
+            return;
+        }
     }
 
     g_warning("Invalid plugin (%s)", filename);
@@ -253,22 +261,15 @@ plugin_system_init(void)
     plugin_matrix = g_hash_table_new_full(g_str_hash, g_int_equal, g_free,
                                           NULL);
 
-    ip_data = g_new0(InputPluginData, 1);
-    op_data = g_new0(OutputPluginData, 1);
-    ep_data = g_new0(EffectPluginData, 1);
-    gp_data = g_new0(GeneralPluginData, 1);
-    vp_data = g_new0(VisPluginData, 1);
-
-
 #ifndef DISABLE_USER_PLUGIN_DIR
-    scan_plugins(bmp_user_plugin_dir);
+    scan_plugins(bmp_paths[BMP_PATH_USER_PLUGIN_DIR]);
     /*
      * This is in a separate loop so if the user puts them in the
      * wrong dir we'll still get them in the right order (home dir
      * first)                                                - Zinx
      */
     while (plugin_dir_list[dirsel]) {
-        dir = g_build_filename(bmp_user_plugin_dir,
+        dir = g_build_filename(bmp_paths[BMP_PATH_USER_PLUGIN_DIR],
                                plugin_dir_list[dirsel++], NULL);
         scan_plugins(dir);
         g_free(dir);
@@ -282,22 +283,22 @@ plugin_system_init(void)
         g_free(dir);
     }
 
-    op_data->output_list = g_list_sort(op_data->output_list, outputlist_compare_func);
-    if (!op_data->current_output_plugin
-        && g_list_length(op_data->output_list)) {
-        op_data->current_output_plugin = op_data->output_list->data;
+    op_data.output_list = g_list_sort(op_data.output_list, outputlist_compare_func);
+    if (!op_data.current_output_plugin
+        && g_list_length(op_data.output_list)) {
+        op_data.current_output_plugin = op_data.output_list->data;
     }
 
-    ip_data->input_list = g_list_sort(ip_data->input_list, inputlist_compare_func);
+    ip_data.input_list = g_list_sort(ip_data.input_list, inputlist_compare_func);
 
-    ep_data->effect_list = g_list_sort(ep_data->effect_list, effectlist_compare_func);
-    ep_data->enabled_list = NULL;
+    ep_data.effect_list = g_list_sort(ep_data.effect_list, effectlist_compare_func);
+    ep_data.enabled_list = NULL;
 
-    gp_data->general_list = g_list_sort(gp_data->general_list, generallist_compare_func);
-    gp_data->enabled_list = NULL;
+    gp_data.general_list = g_list_sort(gp_data.general_list, generallist_compare_func);
+    gp_data.enabled_list = NULL;
 
-    vp_data->vis_list = g_list_sort(vp_data->vis_list, vislist_compare_func);
-    vp_data->enabled_list = NULL;
+    vp_data.vis_list = g_list_sort(vp_data.vis_list, vislist_compare_func);
+    vp_data.enabled_list = NULL;
 
     general_enable_from_stringified_list(cfg.enabled_gplugins);
     vis_enable_from_stringified_list(cfg.enabled_vplugins);
@@ -312,7 +313,7 @@ plugin_system_init(void)
     g_free(cfg.enabled_eplugins);
     cfg.enabled_eplugins = NULL;
 
-    for (node = op_data->output_list; node; node = g_list_next(node)) {
+    for (node = op_data.output_list; node; node = g_list_next(node)) {
         op = OUTPUT_PLUGIN(node->data);
         /*
          * Only test basename to avoid problems when changing
@@ -320,12 +321,12 @@ plugin_system_init(void)
          * basename, so this is usually what the user want.
          */
         if (!strcmp(g_basename(cfg.outputplugin), g_basename(op->filename)))
-            op_data->current_output_plugin = op;
+            op_data.current_output_plugin = op;
         if (op->init)
             op->init();
     }
 
-    for (node = ip_data->input_list; node; node = g_list_next(node)) {
+    for (node = ip_data.input_list; node; node = g_list_next(node)) {
         ip = INPUT_PLUGIN(node->data);
         if (ip->init)
             ip->init();
@@ -356,6 +357,8 @@ plugin_system_cleanup(void)
     VisPlugin *vp;
     GList *node;
 
+    g_message("Shutting down plugin system");
+
     if (bmp_playback_get_playing())
         bmp_playback_stop();
 
@@ -369,17 +372,17 @@ plugin_system_cleanup(void)
         }
         g_module_close(ip->handle);
     }
-    if (ip_data->input_list)
-        g_list_free(ip_data->input_list);
-    g_free(ip_data);
+
+    if (ip_data.input_list)
+        g_list_free(ip_data.input_list);
 
     for (node = get_output_list(); node; node = g_list_next(node)) {
         op = OUTPUT_PLUGIN(node->data);
         g_module_close(op->handle);
     }
-    if (op_data->output_list)
-        g_list_free(op_data->output_list);
-    g_free(op_data);
+    
+    if (op_data.output_list)
+        g_list_free(op_data.output_list);
 
     for (node = get_effect_list(); node; node = g_list_next(node)) {
         ep = EFFECT_PLUGIN(node->data);
@@ -388,54 +391,50 @@ plugin_system_cleanup(void)
             GDK_THREADS_LEAVE();
             while (g_main_iteration(FALSE));
             GDK_THREADS_ENTER();
-
         }
         g_module_close(ep->handle);
     }
-    if (ep_data->effect_list)
-        g_list_free(ep_data->effect_list);
-    g_free(ep_data);
+
+    if (ep_data.effect_list)
+        g_list_free(ep_data.effect_list);
 
     for (node = get_general_enabled_list(); node; node = g_list_next(node)) {
         gp = GENERAL_PLUGIN(node->data);
-        enable_general_plugin(g_list_index(gp_data->general_list, gp), FALSE);
+        enable_general_plugin(g_list_index(gp_data.general_list, gp), FALSE);
     }
-    if (gp_data->enabled_list)
-        g_list_free(gp_data->enabled_list);
+
+    if (gp_data.enabled_list)
+        g_list_free(gp_data.enabled_list);
 
     GDK_THREADS_LEAVE();
-
     while (g_main_iteration(FALSE));
-
     GDK_THREADS_ENTER();
 
     for (node = get_general_list(); node; node = g_list_next(node)) {
         gp = GENERAL_PLUGIN(node->data);
         g_module_close(gp->handle);
     }
-    if (gp_data->general_list)
-        g_list_free(gp_data->general_list);
+
+    if (gp_data.general_list)
+        g_list_free(gp_data.general_list);
 
     for (node = get_vis_enabled_list(); node; node = g_list_next(node)) {
         vp = VIS_PLUGIN(node->data);
-        enable_vis_plugin(g_list_index(vp_data->vis_list, vp), FALSE);
+        enable_vis_plugin(g_list_index(vp_data.vis_list, vp), FALSE);
     }
-    if (vp_data->enabled_list)
-        g_list_free(vp_data->enabled_list);
+
+    if (vp_data.enabled_list)
+        g_list_free(vp_data.enabled_list);
 
     GDK_THREADS_LEAVE();
-
     while (g_main_iteration(FALSE));
-
     GDK_THREADS_ENTER();
 
-    node = get_vis_list();
     for (node = get_vis_list(); node; node = g_list_next(node)) {
         vp = VIS_PLUGIN(node->data);
         g_module_close(vp->handle);
     }
-    if (vp_data->vis_list)
-        g_list_free(vp_data->vis_list);
-    g_free(vp_data);
 
+    if (vp_data.vis_list)
+        g_list_free(vp_data.vis_list);
 }
