@@ -1,5 +1,8 @@
-/*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
+/*  BMP - Cross-platform multimedia player
+ *  Copyright (C) 2003-2004  BMP development team.
+ *
+ *  Based on XMMS:
+ *  Copyright (C) 1998-2003  XMMS development team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,17 +18,26 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
 #ifndef SBUTTON_H
 #define SBUTTON_H
 
-typedef struct {
+#include <glib.h>
+#include <gdk/gdk.h>
+
+#include "widget.h"
+
+#define SBUTTON(x)  ((SButton *)(x))
+struct _SButton {
     Widget sb_widget;
     gint sb_pressed, sb_inside;
     void (*sb_push_cb) (void);
-} SButton;
+};
+
+typedef struct _SButton SButton;
 
 SButton *create_sbutton(GList ** wlist, GdkPixmap * parent, GdkGC * gc,
-			gint x, gint y, gint w, gint h, void (*cb) (void));
+                        gint x, gint y, gint w, gint h, void (*cb) (void));
 void free_sbutton(SButton * b);
 
 #endif
