@@ -270,13 +270,19 @@ vis_set_window(Vis * vis, GdkWindow * window)
 }
 
 Vis *
-create_vis(GList ** wlist, GdkPixmap * parent, GdkWindow * window,
-           GdkGC * gc, gint x, gint y, gint width)
+create_vis(GList ** wlist,
+           GdkPixmap * parent,
+           GdkWindow * window,
+           GdkGC * gc,
+           gint x, gint y,
+           gint width)
 {
     Vis *vis;
 
     vis = g_new0(Vis, 1);
+
     widget_init(&vis->vs_widget, parent, gc, x, y, width, 16, 1);
-    add_widget(wlist, vis);
+    widget_list_add(wlist, WIDGET(vis));
+
     return vis;
 }
