@@ -46,6 +46,6 @@ gchar *vis_stringify_enabled_list(void);
 void vis_enable_from_stringified_list(gchar * list);
 void vis_send_data(gint16 pcm_data[2][512], gint nch, gint length);
 
-extern VisPluginData *vp_data;
+extern VisPluginData vp_data;
 
 #endif
