@@ -62,17 +62,24 @@ struct _Widget {
 
 void widget_init(Widget * widget, GdkPixmap * parent, GdkGC * gc,
                  gint x, gint y, gint width, gint height, gint visible);
+
 void widget_set_position(Widget * widget, gint x, gint y);
 void widget_set_size(Widget * widget, gint width, gint height);
 void widget_queue_redraw(Widget * widget);
 
-gboolean inside_widget(gint x, gint y, gpointer w);
-void show_widget(gpointer w);
-void hide_widget(gpointer w);
-void resize_widget(gpointer w, gint width, gint height);
-void move_widget(gpointer w, gint x, gint y);
-void draw_widget(gpointer w);
-void add_widget(GList ** list, gpointer v);
+void widget_lock(Widget * widget);
+void widget_unlock(Widget * widget);
+
+gboolean widget_contains(Widget * widget, gint x, gint y);
+
+void widget_show(Widget * widget);
+void widget_hide(Widget * widget);
+gboolean widget_is_visible(Widget * widget);
+
+void widget_resize(Widget * widget, gint width, gint height);
+void widget_move(Widget * widget, gint x, gint y);
+void widget_draw(Widget * widget);
+
 void handle_press_cb(GList * wlist, GtkWidget * widget,
                      GdkEventButton * event);
 void handle_release_cb(GList * wlist, GtkWidget * widget,
@@ -81,12 +88,14 @@ void handle_motion_cb(GList * wlist, GtkWidget * widget,
                       GdkEventMotion * event);
 void handle_scroll_cb(GList * wlist, GtkWidget * widget,
                       GdkEventScroll * event);
-void draw_widget_list(GList * wlist, gboolean * redraw, gboolean force);
-void widget_list_change_pixmap(GList * wlist, GdkPixmap * pixmap);
-void clear_widget_list_redraw(GList * wlist);
-void lock_widget(gpointer w);
-void unlock_widget(gpointer w);
-void lock_widget_list(GList * wlist);
-void unlock_widget_list(GList * wlist);
+
+void widget_list_add(GList ** list, Widget * widget);
+void widget_list_draw(GList * list, gboolean * redraw, gboolean force);
+void widget_list_change_pixmap(GList * list, GdkPixmap * pixmap);
+void widget_list_clear_redraw(GList * list);
+void widget_list_lock(GList * list);
+void widget_list_unlock(GList * list);
+
+
 
 #endif
